import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Box, Button, FormControl, FormLabel, Grid, IconButton, Input, Textarea, Typography, } from "@mui/joy";
import { useTranslation } from "react-i18next";
import i18n from "../i18n";
import { Dialog, DialogActions, DialogContent, DialogContentText, DialogTitle, } from "@mui/material";
import * as Sentry from "@sentry/react";
import { useEffect, useState } from "react";
import { nanoid } from "@reduxjs/toolkit";
import { NerdFileUpload } from "../nerdFileUpload";
import { Clear, FilePresent } from "@mui/icons-material";
import * as yup from "yup";
import { useDispatch } from "react-redux";
import { nerdFeedbackErrorSelector, nerdFeedbackOpenSelector, setOpenFeedbackDialog, } from "../redux/nerdFeedback";
import { useSelector } from "react-redux";
/**
 *
 * @param {NerdFeedbackProps} props props
 * @returns {ReactElement} NerdFeedback
 */
export default function NerdFeedback(props) {
    const { t } = useTranslation("components", { i18n });
    const { commentTextareaProps, cancelButtonProps, sendButtonProps, emailInputProps, nameInputProps, user, nerdFileUploadProps, } = props;
    const [name, setName] = useState(user?.name || "");
    const [email, setEmail] = useState(user?.email || "");
    const [comments, setComments] = useState("");
    const [file, setFile] = useState();
    const [isEmailValid, setIsEmailValid] = useState(true);
    const dispatch = useDispatch();
    const open = useSelector(nerdFeedbackOpenSelector);
    const error = useSelector(nerdFeedbackErrorSelector);
    useEffect(() => {
        if (email) {
            nerdEmailValidator(email).then((isValid) => setIsEmailValid(isValid ? true : false));
        }
        else
            setIsEmailValid(true);
    }, [email]);
    const onClose = () => {
        dispatch(setOpenFeedbackDialog({ open: false, error: undefined }));
        setIsEmailValid(true);
        setComments("");
        setName(user?.name || "");
        setEmail(user?.email || "");
        setFile(undefined);
    };
    const send = async () => {
        let data;
        if (file) {
            data = await file.arrayBuffer();
        }
        Sentry.withScope(function (scope) {
            scope.setTag("userFeedback", true);
            scope.setLevel(error ? "error" : "info");
            scope.setFingerprint([
                typeof error === "object" &&
                    error != null &&
                    "uuid" in error &&
                    error.uuid
                    ? error.uuid
                    : nanoid(),
            ]);
            if (data && file) {
                scope.addAttachment({
                    data: new Uint8Array(data),
                    filename: file.name,
                    contentType: file.type,
                });
            }
            if (name && email) {
                let eventId;
                if (typeof error === "object" && error !== null && error !== undefined)
                    eventId = Sentry.captureException(new Error("User Feedback + backend error"), { extra: error });
                else
                    eventId = Sentry.captureMessage("User Feedback");
                const userFeedback = {
                    event_id: eventId,
                    name,
                    email,
                    comments,
                };
                Sentry.captureUserFeedback(userFeedback);
            }
        });
    };
    return (_jsxs(Dialog, { fullWidth: true, open: open, onClose: onClose, children: [_jsx(DialogTitle, { children: t("nerdFeedback.feedback") }), _jsxs(DialogContent, { children: [_jsx(DialogContentText, { children: error !== undefined
                            ? t("nerdFeedback.feedback_description_with_error")
                            : t("nerdFeedback.feedback_description") }), _jsxs(Grid, { mt: 2, container: true, spacing: 1, children: [_jsx(Grid, { xs: 12, children: _jsxs(FormControl, { required: true, children: [_jsx(FormLabel, { children: t("nerdFeedback.name") }), _jsx(Input, { ...nameInputProps, value: name, onChange: (e) => setName(e.target.value) })] }) }), _jsx(Grid, { xs: 12, children: _jsxs(FormControl, { required: true, children: [_jsx(FormLabel, { children: t("nerdFeedback.email") }), _jsx(Input, { ...emailInputProps, value: email, onChange: (e) => setEmail(e.target.value) })] }) }), _jsx(Grid, { xs: 12, children: _jsxs(FormControl, { required: error !== undefined ? false : true, children: [_jsx(FormLabel, { children: t("nerdFeedback.comments") }), _jsx(Textarea, { minRows: 4, ...commentTextareaProps, value: comments, onChange: (e) => setComments(e.target.value) })] }) }), _jsx(Grid, { xs: 12, children: _jsxs(FormControl, { children: [_jsx(FormLabel, { children: t("nerdFeedback.attachment") }), file && (_jsx(Box, { mb: 2, display: "flex", justifyContent: "center", children: _jsxs(Box, { maxWidth: 200, children: [_jsx(IconButton, { sx: {
                                                            position: "relative",
                                                            left: "60%",
                                                            top: 20,
                                                        }, size: "sm", variant: "plain", color: "danger", onClick: () => setFile(undefined), children: _jsx(Clear, {}) }), _jsx(Typography, { level: "h3", color: "primary", textAlign: "center", children: _jsx(FilePresent, {}) }), _jsx(Typography, { level: "body-sm", textAlign: "center", children: file.name })] }) })), _jsx(NerdFileUpload, { ...nerdFileUploadProps, multiple: false, uploadFile: (file) => setFile(file) })] }) })] })] }), _jsxs(DialogActions, { children: [_jsx(Button, { variant: "soft", color: "neutral", ...cancelButtonProps, onClick: (e) => {
                            cancelButtonProps?.onClick?.(e);
                            onClose();
                        }, children: t("common:global.cancel") }), _jsx(Button, { ...sendButtonProps, disabled: !name ||
                            !email ||
                            (error === undefined && !comments) ||
                            !isEmailValid
                            ? true
                            : false, onClick: (e) => {
                            sendButtonProps?.onClick?.(e);
                            send();
                            onClose();
                        }, children: t("nerdFeedback.send") })] })] }));
}
const nerdEmailValidator = (email) => {
    return yup
        .string()
        .email()
        .required()
        .validate(email)
        .then(function () {
        return true;
    })
        .catch(function (err) {
        if (err.name === "ValidationError") {
            return false;
        }
        else {
            console.error(err);
            return;
        }
    });
};
